import sys
import cce

CREATED = cce.CCE_CREATED
DESTROYED = cce.CCE_DESTROYED
MODIFIED = cce.CCE_MODIFIED
SUCCESS = cce.CCE_SUCCESS
DEFER = cce.CCE_DEFER
FAIL = cce.CCE_FAIL
NONE = cce.CCE_NONE       

class CCEObject:
  def __init__(self, cce0, oid):
    self.cce = cce0
    self.oid = oid
  def __getitem__(self, key):
    return self.cce._get(self.oid)[key]
  def __setitem__(self, key, val):
    self.cce._set(self.oid, "", { key: val })
  def __repr__(self):
    msg = "<CCE Object"
    dct = self.cce._get(self.oid)
    for (key, val) in dct.items():
      msg = "%s, %s: \"%s\"" % (msg, key, val)
    msg = msg + ">"
    return msg

class CCE:
  def __init__(self, opts = {}, sktfile = "/usr/sausalito/cced.socket"):
    self.handle = cce.cce_handle_new()
    try:
      self.opt_namespace = opts["Namespace"]
    except KeyError:
      self.opt_namespace = ""
    try:
      self.domain = opts["Domain"]
    except KeyError:
      self.domain = ""
    self.connect(sktfile)
  def connect(self, sktfile):
    return cce.cce_connect_cmnd(self.handle, sktfile)
  def connect_handler(self):
    return cce.cce_connect_handler_cmnd(self.handle)
  def bye(self, status = "NOT_SPECIFIED", message = ""):
    if status == "NOT_SPECIFIED":
      return cce.cce_bye_cmnd(self.handle)
    else:
      return self.bye_handler(status, message)
  def _get(self, oid):
    return cce.cce_get_cmnd(self.handle, oid, self.opt_namespace)
  def auth(self, user, passwd):
    return cce.cce_auth_cmnd(self.handle, user, passwd)
  def authkey(self, user, sessionid):
    return cce.cce_authkey_cmnd(self.handle, user, sessionid)
  def create(self, classname, props):
    return cce.cce_create_cmnd(self.handle, classname, props)
  def _destroy(self, oid):
    return cce.cce_destroy_cmnd(self.handle, oid)
  def destroy(self, obj):
    oid = obj.oid
    del obj
    return self._destroy(self, oid)
  def _set(self, oid, namespace, props):
    return cce.cce_set_cmnd(self.handle, oid, namespace, props)
  def commit(self):
    return cce.cce_commit_cmnd(self.handle)
  def _names_oid(self, oid):
    return cce.cce_names_oid_cmnd(self.handle, oid)
  def names_class(self, classname):
    return cce.cce_names_class_cmnd(self.handle, classname)
  def _find(self, classname, props = {}):
    return cce.cce_find_cmnd(self.handle, classname, props)
  def find(self, classname, props = {}):
    return [ CCEObject(self, oid) for oid in \
     self._find(classname, props) ]
  def _find_sorted(self, classname, props = {}):
    return cce.cce_find_sorted_cmnd(self.handle, classname, props)
  def find_sorted(self, classname, props = {}):
    return [ CCEObject(self, oid) for oid in \
     self._find_sorted(classname, props) ]
  def endkey(self):
    return cce.cce_endkey_cmnd(self.handle)
  def whoami(self):
    return cce.cce_whoami_cmnd(self.handle)
  def last_errors(self):
    return cce.cce_last_errors_cmnd(self.handle)
  def bye_handler(self, status, message = ""):
    return cce.cce_bye_handler_cmnd(self.handle, status, message)
  def _bad_data(self, oid, namespace, key, reason):
    return cce.cce_bad_data_cmnd(self, oid, namespace, key, reason)

if __name__ == "__main__":
  cce0 = CCE()
  print "d0:", cce0.auth("admin", "admin")
  for obj in cce0.find("Inittab"):
    print obj
    if obj["defrunlevel"] == "3":
      obj["defrunlevel"] = "5"
    else:
      obj["defrunlevel"] = "3"
  print "d1:", obj["defrunlevel"]
  cce0.bye()
